<?php
/*
Plugin Name: Follow/Nofollow Control
Plugin URI: http://wikibiz.vn
Description: You can set default to all your external links on your posts are follow or nofollow. You can exclude some domains you want. If you have trouble, please go to the <a href="http://wikibiz.vn/wp/plugins/">Plugin Site</a>.
Version: 1.0
Author: Tran Cuong
Author URI: http://wikibiz.vn
License: GPLv2 
	Copyright 2012 Wikibiz (wikibiz.vn)

*/

require_once ('simplehtmldom_1_5/simple_html_dom.php');

/*----------------------------------------------
	Option Pages 
/*----------------------------------------------*/

/* add plugin's options to white list / defaults */
function nofplg_options_init() { 
	register_setting( 'nofplg_options_options', 'nofplg_item' );
	// if option doesn't exist, set defaults
	if( !get_option( 'nofplg_item' ) ) add_option( 'nofplg_item', array( 'follow' => '', 'nofollow' => '','default'=>1 ), '', 'no' ); 
}

/* add link to plugin's settings page under 'settings' on the admin menu */
function nofplg_options_add_page() { 
	add_options_page( 'Follow-Nofollow Control', 'Follow-Nofollow', 'manage_options', 'follow-nofollow-control', 'nofplg_options_do_page' );
}

/* draw the settings page itself */
function nofplg_options_do_page() { 
	?>
	<div class="wrap">
    <div class="icon32" id="icon-options-general"><br /></div>
		<h2>Follow/Nofollow Control Settings</h2>
		<div style="background:#EDFCF7; padding:7px 10px; border:1px solid #8AAA88; margin:10px 0; width:70%;">
			<ul>
				<li>You can set the default follow or nofollow for all external links of your site.</li>
				<li>If you set follow by default, the only links in the nofollow domains option have rel nofollow</li>
				<li>If you set nofollow by default, the only links in the follow domains option have rel follow</li>
				<li>Internal links of your site are always follow</li>
			</ul>
			<div style="clear:both;"></div>
		</div><!-- end tutorial -->
		<form method="post" action="options.php">
			<?php settings_fields( 'nofplg_options_options' ); // nonce settings page ?>
			<?php $options = get_option( 'nofplg_item' ); // populate $options array from database ?>

			<table class="form-table">
                
<!-- Follow Domains -->
                <tr valign="top">
					<th scope="row">
						<label for="nofplg_item[follow]"><strong>Follow Domains</strong></label>
						<p style="color:#999">The domain you want your external links are being set to follow, for example: your blogs, your another site...</p>
						<p style="color:#999">Please write domain only, eg, wordpress.org, facebook.com..., <strong>not starting by http</strong></p>
					</th>
					<td><textarea style="width:400px; height:200px; border:1px solid #CCC;" id="nofplg_item[follow]" name="nofplg_item[follow]"><?php echo $options['follow']; ?></textarea></td>
                </tr>
                
                
<!-- Nofollow Domains -->
                <tr valign="top">
					<th scope="row">
						<label for="nofplg_item[nofollow]"><strong>Nofollow Domains</strong></label>
						<p style="color:#999">The domain you want your external links are being set to nofollow, for example: spam sites, bad sites, dangerous sites...</p>
						<p style="color:#999">Please write domain only, eg, wordpress.org, facebook.com..., <strong>not starting by http</strong></p>
					</th>
					<td><textarea style="width:400px; height:200px; border:1px solid #CCC;" id="nofplg_item[nofollow]" name="nofplg_item[nofollow]"><?php echo $options['nofollow'];?></textarea></td>
                </tr>

<!-- Default -->
				
				<tr valign="top">
					<th scope="row"><strong>Default</strong> <span style="cursor:help" title="All your external links will be set according to the default except above options">(?)</span></th>
					<td><label for="nofplg_default_follow">							
							<input type="radio" id="nofplg_default_follow" name="nofplg_item[default]" value="1" <?php echo ($options['default'] ? 'checked="checked"' : '');?> />
							<span>Follow</span>
						</label>
						
						<label for="nofplg_default_nofollow" style="margin-left:20px">							
							<input type="radio" id="nofplg_default_nofollow" name="nofplg_item[default]" value="0" <?php echo ($options['default'] ? '' : 'checked="checked"');?> />
							<span>Nofollow</span>
						</label>
					</td>
                </tr>
          
			</table>
			<p class="submit">
				<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
			</p>
		</form>
	</div>
	<?php
}

// add plugin's options to white list on admin initialization
add_action('admin_init', 'nofplg_options_init' ); 

// add link to plugin's settings page in 'settings' menu on admin menu initilization
add_action('admin_menu', 'nofplg_options_add_page'); 

// change the content
add_filter('the_content','nofplg_display_content');

/*----------------------------------------------
	Display content
/*----------------------------------------------*/

function nofplg_display_content($content){
	$html = str_get_html($content);	
	$options = get_option( 'nofplg_item' );
	
	$site_url = get_bloginfo('url'); $parse_site = parse_url($site_url); $site_host = $parse_site['host'];
	
	$follow = $options['follow']; $follow_epl = explode(",",$follow); $follows = array();
	foreach ($follow_epl as $fl) {$follows[] = trim($fl);}	
	
	$nofollow = $options ['nofollow']; $nofollow_epl = explode(",",$nofollow); $nofollows = array();
	foreach ($nofollow_epl as $fl) {$nofollows[] = trim($fl);}	
	
	foreach ($html->find("a") as $a){
		$rel = $a->rel;
		$href = $a->href;
		$parse = parse_url($href); $host = $parse['host'];
		
		if ($options['default']){		// the case default follow			
				if(in_array($host,$nofollows)) $a->rel = 'nofollow';
				else $a->rel = 'follow';
			}
		else {		// the case default nofollow
				if(in_array($host,$follows) || $host == $site_host || $host =='') $a->rel = 'dofollow';
				else $a->rel = 'nofollow';		
		}
	
	}	// endforeach 
	
	return $html->save();
	
}	// end function nofplg display content